@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+ * Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
  * Use of this file is governed by the BSD 3-clause license that
  * can be found in the LICENSE.txt file in the project root.
  */
@@ -141,6 +141,24 @@ public class TestTokenTypeAssignment extends BaseJavaToolTest {
 		assertEquals("'\\n'", literals.toArray()[0]);
 	}
 
+	@Test public void testParserCharLiteralWithBasicUnicodeEscape() throws Exception {
+		Grammar g = new Grammar(
+				"grammar t;\n"+
+				"a : '\\uABCD';\n");
+		Set<?> literals = g.stringLiteralToTypeMap.keySet();
+		// must store literals how they appear in the antlr grammar
+		assertEquals("'\\uABCD'", literals.toArray()[0]);
+	}
+
+	@Test public void testParserCharLiteralWithExtendedUnicodeEscape() throws Exception {
+		Grammar g = new Grammar(
+				"grammar t;\n"+
+				"a : '\\u{1ABCD}';\n");
+		Set<?> literals = g.stringLiteralToTypeMap.keySet();
+		// must store literals how they appear in the antlr grammar
+		assertEquals("'\\u{1ABCD}'", literals.toArray()[0]);
+	}
+
 	protected void checkSymbols(Grammar g,
 								String rulesStr,
 								String allValidTokensStr)
