@@ -1,15 +1,17 @@
 /*
- * Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+ * Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
  * Use of this file is governed by the BSD 3-clause license that
  * can be found in the LICENSE.txt file in the project root.
  */
 
 package org.antlr.v4.automata;
 
+import org.antlr.v4.misc.CharSupport;
 import org.antlr.v4.runtime.atn.ATN;
 import org.antlr.v4.runtime.atn.ATNState;
 import org.antlr.v4.runtime.atn.AtomTransition;
 import org.antlr.v4.runtime.atn.BlockEndState;
+import org.antlr.v4.runtime.atn.CodePointTransitions;
 import org.antlr.v4.runtime.atn.DecisionState;
 import org.antlr.v4.runtime.atn.EpsilonTransition;
 import org.antlr.v4.runtime.atn.NotSetTransition;
@@ -96,17 +98,24 @@ public class ATNOptimizer {
 					if (matchTransition instanceof NotSetTransition) {
 						throw new UnsupportedOperationException("Not yet implemented.");
 					}
-					IntervalSet set = matchTransition.label();
-					int minElem = set.getMinElement();
-					int maxElem = set.getMaxElement();
-					for (int k = minElem; k <= maxElem; k++) {
-						if (matchSet.contains(k)) {
-							char setMin = (char) set.getMinElement();
-							char setMax = (char) set.getMaxElement();
-							// TODO: Token is missing (i.e. position in source will not be displayed).
-							g.tool.errMgr.grammarError(ErrorType.CHARACTERS_COLLISION_IN_SET, g.fileName,
-							                           null, (char) minElem + "-" + (char) maxElem, "[" + setMin + "-" + setMax + "]");
-							break;
+					IntervalSet set =  matchTransition.label();
+					List<Interval> intervals = set.getIntervals();
+					int n = intervals.size();
+					for (int k = 0; k < n; k++) {
+						Interval setInterval = intervals.get(k);
+						int a = setInterval.a;
+						int b = setInterval.b;
+						if (a != -1 && b != -1) {
+							for (int v = a; v <= b; v++) {
+								if (matchSet.contains(v)) {
+									// TODO: Token is missing (i.e. position in source will not be displayed).
+									g.tool.errMgr.grammarError(ErrorType.CHARACTERS_COLLISION_IN_SET, g.fileName,
+											null,
+											CharSupport.getANTLRCharLiteralForChar(v),
+											CharSupport.getIntervalSetEscapedString(matchSet));
+									break;
+								}
+							}
 						}
 					}
 					matchSet.addAll(set);
@@ -115,11 +124,11 @@ public class ATNOptimizer {
 				Transition newTransition;
 				if (matchSet.getIntervals().size() == 1) {
 					if (matchSet.size() == 1) {
-						newTransition = new AtomTransition(blockEndState, matchSet.getMinElement());
+						newTransition = CodePointTransitions.createWithCodePoint(blockEndState, matchSet.getMinElement());
 					}
 					else {
 						Interval matchInterval = matchSet.getIntervals().get(0);
-						newTransition = new RangeTransition(blockEndState, matchInterval.a, matchInterval.b);
+						newTransition = CodePointTransitions.createWithCodePointRange(blockEndState, matchInterval.a, matchInterval.b);
 					}
 				}
 				else {
