@@ -1,31 +1,7 @@
 /*
- * [The "BSD license"]
- *  Copyright (c) 2012 Terence Parr
- *  Copyright (c) 2012 Sam Harwell
- *  All rights reserved.
- *
- *  Redistribution and use in source and binary forms, with or without
- *  modification, are permitted provided that the following conditions
- *  are met:
- *
- *  1. Redistributions of source code must retain the above copyright
- *     notice, this list of conditions and the following disclaimer.
- *  2. Redistributions in binary form must reproduce the above copyright
- *     notice, this list of conditions and the following disclaimer in the
- *     documentation and/or other materials provided with the distribution.
- *  3. The name of the author may not be used to endorse or promote products
- *     derived from this software without specific prior written permission.
- *
- *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- *  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
+ * Use of this file is governed by the BSD 3-clause license that
+ * can be found in the LICENSE.txt file in the project root.
  */
 
 package org.antlr.v4.codegen;
@@ -75,7 +51,7 @@ public class OutputModelWalker {
 		this.templates = templates;
 	}
 
-	public ST walk(OutputModelObject omo) {
+	public ST walk(OutputModelObject omo, boolean header) {
 		// CREATE TEMPLATE FOR THIS OUTPUT OBJECT
 		Class<? extends OutputModelObject> cl = omo.getClass();
 		String templateName = cl.getSimpleName();
@@ -83,6 +59,9 @@ public class OutputModelWalker {
 			tool.errMgr.toolError(ErrorType.NO_MODEL_TO_TEMPLATE_MAPPING, cl.getSimpleName());
 			return new ST("["+templateName+" invalid]");
 		}
+
+		if (header) templateName += "Header";
+
 		ST st = templates.getInstanceOf(templateName);
 		if ( st == null ) {
 			tool.errMgr.toolError(ErrorType.CODE_GEN_TEMPLATES_INCOMPLETE, templateName);
@@ -124,7 +103,7 @@ public class OutputModelWalker {
 				Object o = fi.get(omo);
 				if ( o instanceof OutputModelObject ) {  // SINGLE MODEL OBJECT?
 					OutputModelObject nestedOmo = (OutputModelObject)o;
-					ST nestedST = walk(nestedOmo);
+					ST nestedST = walk(nestedOmo, header);
 //					System.out.println("set ModelElement "+fieldName+"="+nestedST+" in "+templateName);
 					st.add(fieldName, nestedST);
 				}
@@ -136,7 +115,7 @@ public class OutputModelWalker {
 					Collection<?> nestedOmos = (Collection<?>)o;
 					for (Object nestedOmo : nestedOmos) {
 						if ( nestedOmo==null ) continue;
-						ST nestedST = walk((OutputModelObject)nestedOmo);
+						ST nestedST = walk((OutputModelObject)nestedOmo, header);
 //						System.out.println("set ModelElement "+fieldName+"="+nestedST+" in "+templateName);
 						st.add(fieldName, nestedST);
 					}
@@ -145,7 +124,7 @@ public class OutputModelWalker {
 					Map<?, ?> nestedOmoMap = (Map<?, ?>)o;
 					Map<Object, ST> m = new LinkedHashMap<Object, ST>();
 					for (Map.Entry<?, ?> entry : nestedOmoMap.entrySet()) {
-						ST nestedST = walk((OutputModelObject)entry.getValue());
+						ST nestedST = walk((OutputModelObject)entry.getValue(), header);
 //						System.out.println("set ModelElement "+fieldName+"="+nestedST+" in "+templateName);
 						m.put(entry.getKey(), nestedST);
 					}
