@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+ * Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
  * Use of this file is governed by the BSD 3-clause license that
  * can be found in the LICENSE.txt file in the project root.
  */
@@ -92,7 +92,7 @@ public class DOTGenerator {
 					if ( target.stateNumber == Integer.MAX_VALUE ) continue;
 					int ttype = i-1; // we shift up for EOF as -1 for parser
 					String label = String.valueOf(ttype);
-					if ( isLexer ) label = "'"+getEdgeLabel(String.valueOf((char) i))+"'";
+					if ( isLexer ) label = "'"+getEdgeLabel(new StringBuilder().appendCodePoint(i).toString())+"'";
 					else if ( grammar!=null ) label = grammar.getTokenDisplayName(ttype);
 					ST st = stlib.getInstanceOf("edge");
 					st.add("label", label);
@@ -259,7 +259,7 @@ public class DOTGenerator {
 					edgeST = stlib.getInstanceOf("edge");
 					AtomTransition atom = (AtomTransition)edge;
 					String label = String.valueOf(atom.label);
-					if ( isLexer ) label = "'"+getEdgeLabel(String.valueOf((char)atom.label))+"'";
+					if ( isLexer ) label = "'"+getEdgeLabel(new StringBuilder().appendCodePoint(atom.label).toString())+"'";
 					else if ( grammar!=null ) label = grammar.getTokenDisplayName(atom.label);
 					edgeST.add("label", getEdgeLabel(label));
 				}
@@ -289,7 +289,8 @@ public class DOTGenerator {
 				edgeST.add("arrowhead", arrowhead);
 				if (s.getNumberOfTransitions() > 1) {
 					edgeST.add("transitionIndex", i);
-				} else {
+				}
+				else {
 					edgeST.add("transitionIndex", false);
 				}
 				dot.add("edges", edgeST);
